﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TransportGuideContracts.BindingModels;
using TransportGuideContracts.BusinessLogicsContracts;
using TransportGuideContracts.SearchModels;
using TransportGuideContracts.StoragesContracts;
using TransportGuideContracts.ViewModels;

namespace BusinessLogic.BusinessLogic
{
	public class TransportTypeLogic : ITransportTypeLogic
	{
		private readonly ILogger _logger;
		private readonly ITransportTypeStorage _transportTypeStorage;

		public TransportTypeLogic(ILogger<TransportTypeLogic> logger, ITransportTypeStorage transportTypeStorage)
		{
			_logger = logger;
			_transportTypeStorage = transportTypeStorage;
		}

		public bool Create(TransportTypeBindingModel model)
		{
			CheckModel(model);
			if (_transportTypeStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}

		public bool Delete(TransportTypeBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_transportTypeStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}

		public TransportTypeViewModel? ReadElement(TransportTypeSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Name:{Name}.Id:{ Id}", model.Name, model.Id);
			var element = _transportTypeStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
			return element;
		}

		public List<TransportTypeViewModel>? ReadList(TransportTypeSearchModel? model)
		{
			_logger.LogInformation("ReadList. Name:{Name}.Id:{ Id}", model?.Name, model?.Id);
			var list = model == null ? _transportTypeStorage.GetFullList() : _transportTypeStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count:{Count}", list.Count);
			return list;
		}

		public bool Update(TransportTypeBindingModel model)
		{
			CheckModel(model);
			if (_transportTypeStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(TransportTypeBindingModel model, bool withParams =
	 true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			if (string.IsNullOrEmpty(model.Name))
			{
				throw new ArgumentNullException("Нет названия ",nameof(model.Name));
			}
			if (model.Price <= 0)
			{
				throw new ArgumentNullException("Цена  должна быть больше 0", nameof(model.Price));
			}
			_logger.LogInformation("TransportType. Name:{Name}. Price:{ Price}. Id: { Id}", model.Name, model.Price, model.Id);
			var element = _transportTypeStorage.GetElement(new TransportTypeSearchModel
			{
				Name = model.Name
			});
			if (element != null && element.Id != model.Id)
			{
				throw new InvalidOperationException("Type с таким названием уже есть");
			}
		}
	}
}
